==> molecule-plugins-23.0.0-1-any.pkg.tar.xz: usr/lib/python3.10/site-packages/molecule_plugins/gce/driver.py <==
#  Copyright (c) 2015-2018 Cisco Systems, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
from typing import Dict
import os
from molecule import logger
from molecule.api import Driver

from molecule import util

LOG = logger.get_logger(__name__)


class GCE(Driver):
    """
    The class responsible for managing `GCE`_ instances.  `GCE`_
    is `not` the default driver used in Molecule.

    GCE is somewhat different than other cloud providers.  There is not
    an Ansible module for managing ssh keys.  This driver assumes the developer
    has deployed project wide ssh key.

    Molecule leverages Ansible's `gce_module`_, by mapping variables from
    ``molecule.yml`` into ``create.yml`` and ``destroy.yml``.

    .. _`gce_module`: https://docs.ansible.com/ansible/latest/gce_module.html

    .. code-block:: yaml

        driver:
          name: gce
        platforms:
          - name: instance

    .. code-block:: bash

        $ pip install molecule[gce]

    Change the options passed to the ssh client.

    .. code-block:: yaml

        driver:
          name: gce
          ssh_connection_options:
            - '-o ControlPath=~/.ansible/cp/%r@%h-%p'

    .. important::

        Molecule does not merge lists, when overriding the developer must
        provide all options.

    Provide a list of files Molecule will preserve, relative to the scenario
    ephemeral directory, after any ``destroy`` subcommand execution.

    .. code-block:: yaml

        driver:
          name: gce
          safe_files:
            - foo

    .. _`GCE`: https://cloud.google.com/compute/docs/
    """  # noqa

    def __init__(self, config=None):
        super(GCE, self).__init__(config)
        self._name = "gce"

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def login_cmd_template(self):
        connection_options = " ".join(self.ssh_connection_options)

        return (
            "ssh {{address}} "
            "-l {{user}} "
            "-p {{port}} "
            "-i {{identity_file}} "
            "{}"
        ).format(connection_options)

    @property
    def default_safe_files(self):
        return [self.instance_config]

    @property
    def default_ssh_connection_options(self):
        return self._get_ssh_connection_options()

    def login_options(self, instance_name):
        d = {"instance": instance_name}

        return util.merge_dicts(d, self._get_instance_config(instance_name))

    def ansible_connection_options(self, instance_name):
        try:
            d = self._get_instance_config(instance_name)

            if "instance_os_type" in d:
                if d["instance_os_type"] == "linux":
                    return {
                        "ansible_user": d["user"],
                        "ansible_host": d["address"],
                        "ansible_port": d["port"],
                        "ansible_private_key_file": d["identity_file"],
                        "ansible_connection": "ssh",
                        "ansible_ssh_common_args": " ".join(
                            self.ssh_connection_options
                        ),
                    }

                if d["instance_os_type"] == "windows":
                    return {
                        "ansible_user": d["user"],
                        "ansible_host": d["address"],
                        "ansible_password": d["password"],
                        "ansible_port": d["port"],
                        "ansible_connection": "winrm",
                        "ansible_winrm_transport": d["winrm_transport"],
                        "ansible_winrm_server_cert_validation": d[
                            "winrm_server_cert_validation"
                        ],
                        "ansible_become_method": "runas",
                    }
        except StopIteration:
            return {}
        except IOError:
            # Instance has yet to be provisioned , therefore the
            # instance_config is not on disk.
            return {}

    def _get_instance_config(self, instance_name):
        instance_config_dict = util.safe_load_file(self._config.driver.instance_config)

        return next(
            item for item in instance_config_dict if item["instance"] == instance_name
        )

    def sanity_checks(self):
        # FIXME(decentral1se): Implement sanity checks
        pass

    def template_dir(self):
        """Return path to its own cookiecutterm templates. It is used by init
        command in order to figure out where to load the templates from.
        """
        return os.path.join(os.path.dirname(__file__), "cookiecutter")

    @property
    def required_collections(self) -> Dict[str, str]:
        # https://galaxy.ansible.com/google/cloud
        return {"google.cloud": "1.0.2", "community.crypto": "1.8.0"}

==> molecule-plugins-23.4.1-3-any.pkg.tar.xz: usr/lib/python3.11/site-packages/molecule_plugins/gce/driver.py <==
#  Copyright (c) 2015-2018 Cisco Systems, Inc.
#
#  Permission is hereby granted, free of charge, to any person obtaining a copy
#  of this software and associated documentation files (the "Software"), to
#  deal in the Software without restriction, including without limitation the
#  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
#  sell copies of the Software, and to permit persons to whom the Software is
#  furnished to do so, subject to the following conditions:
#
#  The above copyright notice and this permission notice shall be included in
#  all copies or substantial portions of the Software.
#
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
import os

from molecule import logger, util
from molecule.api import Driver

LOG = logger.get_logger(__name__)


class GCE(Driver):
    """
    The class responsible for managing `GCE`_ instances.  `GCE`_
    is `not` the default driver used in Molecule.

    GCE is somewhat different than other cloud providers.  There is not
    an Ansible module for managing ssh keys.  This driver assumes the developer
    has deployed project wide ssh key.

    Molecule leverages Ansible's `gce_module`_, by mapping variables from
    ``molecule.yml`` into ``create.yml`` and ``destroy.yml``.

    .. _`gce_module`: https://docs.ansible.com/ansible/latest/gce_module.html

    .. code-block:: yaml

        driver:
          name: gce
        platforms:
          - name: instance

    .. code-block:: bash

        $ pip install molecule[gce]

    Change the options passed to the ssh client.

    .. code-block:: yaml

        driver:
          name: gce
          ssh_connection_options:
            - '-o ControlPath=~/.ansible/cp/%r@%h-%p'

    .. important::

        Molecule does not merge lists, when overriding the developer must
        provide all options.

    Provide a list of files Molecule will preserve, relative to the scenario
    ephemeral directory, after any ``destroy`` subcommand execution.

    .. code-block:: yaml

        driver:
          name: gce
          safe_files:
            - foo

    .. _`GCE`: https://cloud.google.com/compute/docs/
    """

    def __init__(self, config=None) -> None:
        super().__init__(config)
        self._name = "gce"

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def login_cmd_template(self):
        connection_options = " ".join(self.ssh_connection_options)

        return (
            "ssh {{address}} "
            "-l {{user}} "
            "-p {{port}} "
            "-i {{identity_file}} "
            "{}"
        ).format(connection_options)

    @property
    def default_safe_files(self):
        return [self.instance_config]

    @property
    def default_ssh_connection_options(self):
        return self._get_ssh_connection_options()

    def login_options(self, instance_name):
        d = {"instance": instance_name}

        return util.merge_dicts(d, self._get_instance_config(instance_name))

    def ansible_connection_options(self, instance_name):
        try:
            d = self._get_instance_config(instance_name)

            if "instance_os_type" in d:
                if d["instance_os_type"] == "linux":
                    return {
                        "ansible_user": d["user"],
                        "ansible_host": d["address"],
                        "ansible_port": d["port"],
                        "ansible_private_key_file": d["identity_file"],
                        "ansible_connection": "ssh",
                        "ansible_ssh_common_args": " ".join(
                            self.ssh_connection_options,
                        ),
                    }

                if d["instance_os_type"] == "windows":
                    return {
                        "ansible_user": d["user"],
                        "ansible_host": d["address"],
                        "ansible_password": d["password"],
                        "ansible_port": d["port"],
                        "ansible_connection": "winrm",
                        "ansible_winrm_transport": d["winrm_transport"],
                        "ansible_winrm_server_cert_validation": d[
                            "winrm_server_cert_validation"
                        ],
                        "ansible_become_method": "runas",
                    }
        except StopIteration:
            return {}
        except OSError:
            # Instance has yet to be provisioned , therefore the
            # instance_config is not on disk.
            return {}

    def _get_instance_config(self, instance_name):
        instance_config_dict = util.safe_load_file(self._config.driver.instance_config)

        return next(
            item for item in instance_config_dict if item["instance"] == instance_name
        )

    def sanity_checks(self):
        # FIXME(decentral1se): Implement sanity checks
        pass

    def template_dir(self):
        """Return path to its own cookiecutterm templates. It is used by init
        command in order to figure out where to load the templates from.
        """
        return os.path.join(os.path.dirname(__file__), "cookiecutter")

    @property
    def required_collections(self) -> dict[str, str]:
        # https://galaxy.ansible.com/google/cloud
        return {"google.cloud": "1.0.2", "community.crypto": "1.8.0"}
